/* Copyright (C) Daiki Ueno <ueno@unixuser.org>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include "du.h"
#include "rect.h"
#include "heap.h"

static void
rect_print (rect_t rect)
{
  if (rect->rect0)
    rect_print (rect->rect0);
  if (rect->rect1)
    rect_print (rect->rect1);
  if (rect->data)
    printf ("%#.15lf\t%#.15lf\t%#.15lf\t%#.15lf\t%d\t%c\t%s\t%s\n",
	    rect->x, rect->y, rect->width, rect->height,
	    rect->data->depth, rect->data->children ? '1' : '0',
	    rect->data->name, rect->data->path);
}

static void
usage (char *program, FILE *out)
{
  fprintf (out, "Usage: %s [OPTIONS] [DIRECTORY]\n"
	   "Options are\n"
	   "\t--geometry, -g\tSpecify size of the outermost rectangle\n"
	   "\t--help, -h\tShow this help\n", program);
}

int
main (int argc, char **argv)
{
  du_data_t data;
  rect_t rect;
  int c;
  double width = 800.0, height = 600.0;

  struct option long_options[] = {
    {"geometry", 1, 0, 'g'},
    {"help", 0, 0, 'h'},
    {0, 0, 0, 0}
  };

  while (1)
    {
      int option_index = 0;

      c = getopt_long (argc, argv, "g:h", long_options, &option_index);
      if (c == -1)
	break;
      switch (c)
	{
	case 'g':
	  if (sscanf (optarg, "%lfx%lf", &width, &height) != 2)
	    goto error;
	  break;
	case 'h':
	  usage (argv[0], stdout);
	  exit (0);
	default:
	  goto error;
	}
    }

  if (optind == argc)
    data = du_fd (stdin);
  else
    data = du_dir (argv[optind]);
  if (!data)
    {
      perror ("du");
      exit (1);
    }

  rect = rect_new (0.0, 0.0, width, height);
  if (!rect)
    {
      perror ("rect_new");
      exit (1);
    }

  if (fill_rect (data, rect) < 0)
    {
      perror ("fill_rect");
      exit (1);
    }

  rect_print (rect);

  du_data_free (data);
  rect_free (rect);

  return 0;

 error:
  usage (argv[0], stderr);
  exit (1);
}
